//  
//  Copyright (C) 2009 James P Michels III
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;

using Gtk;

using GnomeRDP.Identies;
using GnomeRDP.Sessions;
using GnomeRDP.Logging;
using GnomeRDP.Profiles;
using GnomeRDP.Rdp;
using GnomeRDP.Ssh;
using GnomeRDP.Vnc;

using AppIndicator;

namespace GnomeRDP
{
	public partial class MainWindow: Gtk.Window
	{			
		private ApplicationIndicator applicationIndicator;
		private Gtk.Action actionQuit;
		private Gtk.Action actionToggleVisible;
		private bool hideOnDelete = false;
				
		private const string sessionKey = "Session";
		
		public MainWindow (): base (Gtk.WindowType.Toplevel)
		{
			Build ();
			
			this.DeleteEvent += (s, e) =>
			{
				if (hideOnDelete) 
				{
					Visible = false;
					e.RetVal = true;
				}
			};
			
			this.actionQuit = new Gtk.Action ("QuitAction", "Quit");
			this.actionQuit.Activated += (s, e) => Application.Quit ();
		
			this.actionToggleVisible = new Gtk.Action("ToggleVisibleAction", "Show/Hide");
			this.actionToggleVisible.Activated += (s, e) => { this.Visible = !this.Visible; };

			this.applicationIndicator = new ApplicationIndicator ("gnome-rdp", "gnome-rdp", Category.ApplicationStatus);
			this.applicationIndicator.ConnectionChanged += (s, e) => { hideOnDelete = this.applicationIndicator.Connected; };
			
			this.applicationIndicator.Menu = CreateMenu();
			this.applicationIndicator.Status = Status.Active;
						
			this.Icon = ResourceLoader.Find(ResourceLoader.Icons.gnomeRdp);
		}

		public void UpdateMenu()
		{
			Menu oldMenu = this.applicationIndicator.Menu;
			
			this.applicationIndicator.Menu = CreateMenu();
			
			oldMenu.Dispose();
		}
										
		private Menu CreateMenu()
		{
			Menu topMenu = new Menu();
			
			topMenu.Append(actionToggleVisible.CreateMenuItem());
			topMenu.Append(new SeparatorMenuItem());
				
			foreach (var group in Program.SessionCollection.Groups) 
			{	
				MenuItem groupMenu = new MenuItem(group);
				
				Menu subMenu = new Menu();
				foreach(var session in Program.SessionCollection.Items.Where(s => s.Group == group).OrderBy(s => s.Server))
				{
					MenuItem menuItem = new MenuItem(session.MenuFormat);
					menuItem.TooltipText = session.Tooltip;
					menuItem.Activated += PopupMenuItem_Activated;
					menuItem.Data[sessionKey] = session;
										
					subMenu.Append(menuItem);
				}
				groupMenu.Submenu = subMenu;

				topMenu.Append(groupMenu);
			}

			topMenu.Append(new SeparatorMenuItem());

			foreach (var session in Program.SessionCollection.Items.Where(s => string.IsNullOrEmpty(s.Group)).OrderBy(s => s.Server)) 
			{
				MenuItem menuItem = new MenuItem(session.MenuFormat);
				menuItem.TooltipText = session.Tooltip;
				menuItem.Activated += PopupMenuItem_Activated;
				menuItem.Data[sessionKey] = session;					
				
				topMenu.Append(menuItem);
			}
			
			topMenu.Append(new SeparatorMenuItem());
			topMenu.Append(actionQuit.CreateMenuItem());
			
			topMenu.ShowAll();
			return topMenu;
		}
				
		private void PopupMenuItem_Activated(object sender, EventArgs e)
		{
			try
			{
				MenuItem menuItem = (MenuItem) sender;				
				Session session = (Session)menuItem.Data[sessionKey];
				Program.SessionCollection.Connect(session);
			}
			catch
			{
			}
		}
	}
}